==> siddhi-core-5.1.2-sources.jar: io/siddhi/core/query/processor/stream/window/HopingWindowProcessor.java <==
/*
 * Copyright (c) 2019, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package io.siddhi.core.query.processor.stream.window;

import io.siddhi.core.config.SiddhiQueryContext;
import io.siddhi.core.event.ComplexEvent;
import io.siddhi.core.event.ComplexEventChunk;
import io.siddhi.core.event.stream.MetaStreamEvent;
import io.siddhi.core.event.stream.StreamEvent;
import io.siddhi.core.event.stream.StreamEventCloner;
import io.siddhi.core.event.stream.holder.StreamEventClonerHolder;
import io.siddhi.core.event.stream.populater.ComplexEventPopulater;
import io.siddhi.core.event.stream.populater.SelectiveComplexEventPopulater;
import io.siddhi.core.event.stream.populater.StreamEventPopulaterFactory;
import io.siddhi.core.executor.ExpressionExecutor;
import io.siddhi.core.query.processor.ProcessingMode;
import io.siddhi.core.query.processor.Processor;
import io.siddhi.core.util.config.ConfigReader;
import io.siddhi.core.util.snapshot.state.State;
import io.siddhi.core.util.snapshot.state.StateFactory;
import io.siddhi.query.api.definition.AbstractDefinition;
import io.siddhi.query.api.definition.Attribute;

import java.util.ArrayList;
import java.util.List;

/**
 * Performs event processing in a hopping manner
 *
 * @param <S> current state of the processor
 */
public abstract class HopingWindowProcessor<S extends State> extends WindowProcessor<S> {

    protected List<Attribute> internalAttributes;
    protected HopingTimestampPopulator hopingTimestampPopulator;

    @Override
    protected StateFactory<S> init(MetaStreamEvent metaStreamEvent, AbstractDefinition inputDefinition,
                                   ExpressionExecutor[] attributeExpressionExecutors,
                                   ConfigReader configReader, StreamEventClonerHolder streamEventClonerHolder,
                                   boolean outputExpectsExpiredEvents, boolean findToBeExecuted,
                                   SiddhiQueryContext siddhiQueryContext) {
        StateFactory<S> stateFactory = init(attributeExpressionExecutors, configReader, outputExpectsExpiredEvents,
                siddhiQueryContext);
        Attribute groupingKey = new Attribute("_hopingTimestamp", Attribute.Type.STRING);
        internalAttributes = new ArrayList<Attribute>(1);
        internalAttributes.add(groupingKey);
        metaStreamEvent.addData(groupingKey);
        return stateFactory;
    }

    /**
     * The init method of the WindowProcessor, this method will be called before other methods
     *
     * @param attributeExpressionExecutors the executors of each function parameters
     * @param configReader                 the config reader of window
     * @param outputExpectsExpiredEvents   is expired events sent as output
     * @param siddhiQueryContext           the context of the siddhi query
     */
    protected abstract StateFactory<S> init(ExpressionExecutor[] attributeExpressionExecutors,
                                            ConfigReader configReader,
                                            boolean outputExpectsExpiredEvents,
                                            SiddhiQueryContext siddhiQueryContext);

    protected void processEventChunk(ComplexEventChunk<StreamEvent> streamEventChunk, Processor nextProcessor,
                                     StreamEventCloner streamEventCloner,
                                     ComplexEventPopulater complexEventPopulater, S state) {
        processEventChunk(streamEventChunk, nextProcessor, streamEventCloner, hopingTimestampPopulator);
    }

    /**
     * The main processing method that will be called upon event arrival
     *
     * @param streamEventChunk         the event chunk that need to be processed
     * @param nextProcessor            the next processor to which the success events need to be passed
     * @param streamEventCloner        helps to clone the incoming event for local storage or modification
     * @param hopingTimestampPopulator helps to populate the events with the hoping timestamp
     */
    protected abstract void processEventChunk(ComplexEventChunk<StreamEvent> streamEventChunk,
                                              Processor nextProcessor, StreamEventCloner streamEventCloner,
                                              HopingTimestampPopulator hopingTimestampPopulator);

    public void constructStreamEventPopulater(MetaStreamEvent metaStreamEvent, int streamEventChainIndex) {
        super.constructStreamEventPopulater(metaStreamEvent, streamEventChainIndex);
        if (this.hopingTimestampPopulator == null) {
            hopingTimestampPopulator = new HopingTimestampPopulator(StreamEventPopulaterFactory.constructEventPopulator(
                    metaStreamEvent, streamEventChainIndex, internalAttributes));
        }
    }

    @Override
    public ProcessingMode getProcessingMode() {
        return ProcessingMode.HOP;
    }

    /**
     * Populates hopping timestamp to the complex event
     */
    public class HopingTimestampPopulator {

        private SelectiveComplexEventPopulater selectiveComplexEventPopulater;

        public HopingTimestampPopulator(SelectiveComplexEventPopulater selectiveComplexEventPopulater) {
            this.selectiveComplexEventPopulater = selectiveComplexEventPopulater;
        }

        public void populateComplexEvent(ComplexEvent complexEvent, String key) {
            selectiveComplexEventPopulater.populateComplexEvent(complexEvent, new Object[]{key});
        }

    }
}

==> siddhi-core-5.1.12-sources.jar: io/siddhi/core/query/processor/stream/window/HopingWindowProcessor.java <==
/*
 * Copyright (c) 2019, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package io.siddhi.core.query.processor.stream.window;

import io.siddhi.core.config.SiddhiQueryContext;
import io.siddhi.core.event.ComplexEvent;
import io.siddhi.core.event.ComplexEventChunk;
import io.siddhi.core.event.stream.MetaStreamEvent;
import io.siddhi.core.event.stream.StreamEvent;
import io.siddhi.core.event.stream.StreamEventCloner;
import io.siddhi.core.event.stream.holder.StreamEventClonerHolder;
import io.siddhi.core.event.stream.populater.ComplexEventPopulater;
import io.siddhi.core.event.stream.populater.SelectiveComplexEventPopulater;
import io.siddhi.core.event.stream.populater.StreamEventPopulaterFactory;
import io.siddhi.core.executor.ExpressionExecutor;
import io.siddhi.core.query.processor.ProcessingMode;
import io.siddhi.core.query.processor.Processor;
import io.siddhi.core.util.config.ConfigReader;
import io.siddhi.core.util.snapshot.state.State;
import io.siddhi.core.util.snapshot.state.StateFactory;
import io.siddhi.query.api.definition.AbstractDefinition;
import io.siddhi.query.api.definition.Attribute;

import java.util.ArrayList;
import java.util.List;

/**
 * Performs event processing in a hopping manner
 *
 * @param <S> current state of the processor
 */
public abstract class HopingWindowProcessor<S extends State> extends WindowProcessor<S> {

    protected List<Attribute> internalAttributes;
    protected HopingTimestampPopulator hopingTimestampPopulator;

    @Override
    protected StateFactory<S> init(MetaStreamEvent metaStreamEvent, AbstractDefinition inputDefinition,
                                   ExpressionExecutor[] attributeExpressionExecutors,
                                   ConfigReader configReader, StreamEventClonerHolder streamEventClonerHolder,
                                   boolean outputExpectsExpiredEvents, boolean findToBeExecuted,
                                   SiddhiQueryContext siddhiQueryContext) {
        StateFactory<S> stateFactory = init(attributeExpressionExecutors, configReader, outputExpectsExpiredEvents,
                siddhiQueryContext);
        Attribute groupingKey = new Attribute("_hopingTimestamp", Attribute.Type.STRING);
        internalAttributes = new ArrayList<Attribute>(1);
        internalAttributes.add(groupingKey);
        metaStreamEvent.addData(groupingKey);
        return stateFactory;
    }

    /**
     * The init method of the WindowProcessor, this method will be called before other methods
     *
     * @param attributeExpressionExecutors the executors of each function parameters
     * @param configReader                 the config reader of window
     * @param outputExpectsExpiredEvents   is expired events sent as output
     * @param siddhiQueryContext           the context of the siddhi query
     */
    protected abstract StateFactory<S> init(ExpressionExecutor[] attributeExpressionExecutors,
                                            ConfigReader configReader,
                                            boolean outputExpectsExpiredEvents,
                                            SiddhiQueryContext siddhiQueryContext);

    protected void processEventChunk(ComplexEventChunk<StreamEvent> streamEventChunk, Processor nextProcessor,
                                     StreamEventCloner streamEventCloner,
                                     ComplexEventPopulater complexEventPopulater, S state) {
        streamEventChunk.reset();
        while (streamEventChunk.hasNext()) {
            StreamEvent streamEvent = streamEventChunk.next();
            streamEvent.setBeforeWindowData(null);
        }
        streamEventChunk.reset();
        processEventChunk(streamEventChunk, nextProcessor, streamEventCloner, hopingTimestampPopulator);
    }

    /**
     * The main processing method that will be called upon event arrival
     *
     * @param streamEventChunk         the event chunk that need to be processed
     * @param nextProcessor            the next processor to which the success events need to be passed
     * @param streamEventCloner        helps to clone the incoming event for local storage or modification
     * @param hopingTimestampPopulator helps to populate the events with the hoping timestamp
     */
    protected abstract void processEventChunk(ComplexEventChunk<StreamEvent> streamEventChunk,
                                              Processor nextProcessor, StreamEventCloner streamEventCloner,
                                              HopingTimestampPopulator hopingTimestampPopulator);

    public void constructStreamEventPopulater(MetaStreamEvent metaStreamEvent, int streamEventChainIndex) {
        super.constructStreamEventPopulater(metaStreamEvent, streamEventChainIndex);
        if (this.hopingTimestampPopulator == null) {
            hopingTimestampPopulator = new HopingTimestampPopulator(StreamEventPopulaterFactory.constructEventPopulator(
                    metaStreamEvent, streamEventChainIndex, internalAttributes));
        }
    }

    @Override
    public ProcessingMode getProcessingMode() {
        return ProcessingMode.HOP;
    }

    /**
     * Populates hopping timestamp to the complex event
     */
    public class HopingTimestampPopulator {

        private SelectiveComplexEventPopulater selectiveComplexEventPopulater;

        public HopingTimestampPopulator(SelectiveComplexEventPopulater selectiveComplexEventPopulater) {
            this.selectiveComplexEventPopulater = selectiveComplexEventPopulater;
        }

        public void populateComplexEvent(ComplexEvent complexEvent, String key) {
            selectiveComplexEventPopulater.populateComplexEvent(complexEvent, new Object[]{key});
        }

    }
}
